/****************************************************************************
**
** Copyright (C) 2006 Trolltech ASA. All rights reserved.
**
** This file is part of the documentation of Qt. It was originally
** published as part of Qt Quarterly.
**
** This file may be used under the terms of the GNU General Public License
** version 2.0 as published by the Free Software Foundation or under the
** terms of the Qt Commercial License Agreement. The respective license
** texts for these are provided with the open source and commercial
** editions of Qt.
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************/

#include "moviewidget.h"
#include <QtGui/QLayout>
#include <Carbon/Carbon.h>
#include <QuickTime/QuickTime.h>

MovieWidget::MovieWidget(QWidget *parent)
    : QWidget(parent), currentMovie(0)
{
    HIViewRef movieView;
    HIObjectCreate(kHIMovieViewClassID, 0, reinterpret_cast<HIObjectRef*>(&movieView));
    HIMovieViewChangeAttributes(movieView,
                    kHIMovieViewAutoIdlingAttribute | kHIMovieViewControllerVisibleAttribute, 0);

    create(WId(movieView));
    setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
}

MovieWidget::~MovieWidget()
{
    if (currentMovie)
        DisposeMovie(currentMovie);
}

void MovieWidget::loadMovie(const QString &fileName)
{
    QTVisualContextRef visualContext = 0;
    Boolean active = true;
    DataReferenceRecord dataRef;
    Movie oldMovie = 0;

    // create an array of properties for NewMovieFromProperties
    // these properties describe how to create the new movie
    // see Movies.h for the full list of available properties
    QTNewMoviePropertyElement newMovieProperties[] = {
        {kQTPropertyClass_DataLocation,     kQTDataLocationPropertyID_DataReference, sizeof(dataRef),       &dataRef,       0},
        {kQTPropertyClass_NewMovieProperty, kQTNewMoviePropertyID_Active,            sizeof(active),        &active,        0},
        {kQTPropertyClass_Context,          kQTContextPropertyID_VisualContext,      sizeof(visualContext), &visualContext, 0},
    };

    CFStringRef cfString = CFStringCreateWithCharacters(0,
                                    reinterpret_cast<const UniChar *>(fileName.unicode()),
                                    fileName.length());
    QTNewDataReferenceFromFullPathCFString(cfString, kQTPOSIXPathStyle, 0, &dataRef.dataRef, &dataRef.dataRefType);
    CFRelease(cfString);


    // if there's an old movie save it so we can dispose of it if
    // we load up a new movie successfully
    oldMovie = currentMovie;
    currentMovie = 0;

    // create a new movie using movie properties
    // when calling this function, you supply a set of input properties that describe the information
    // required to instantiate the movie (its data reference, audio context, visual context, and so on)
    NewMovieFromProperties(sizeof(newMovieProperties) / sizeof(newMovieProperties[0]), // the number of properties in the array passed in inputProperties
            newMovieProperties,    // a pointer to a property array describing how to instantiate the movie
            0,                        // number of properties in the array passed in outputProperties
            0,                    // pointer to a property array to receive output parameters - 0 if you don’t want this information
            &currentMovie);  // pointer to a variable that receives the new movie
    // make sure to dispose of the dataRef we no longer need it
    DisposeHandle(dataRef.dataRef);

    // set the HIMovieView's current movie
    HIMovieViewSetMovie(HIViewRef(winId()), currentMovie);

    if (oldMovie != 0)
        DisposeMovie(oldMovie);
    // new movie has a new size, need to update.
    updateGeometry();
    emit optimalSizeChange();
}

QSize MovieWidget::sizeHint() const
{
    EventRef event;
    HIRect optimalBounds;
    CreateEvent(0, kEventClassControl, kEventControlGetOptimalBounds,
                GetCurrentEventTime(), kEventAttributeUserEvent, &event);
    SendEventToEventTargetWithOptions(event,
            HIObjectGetEventTarget(HIObjectRef(winId())),
            kEventTargetDontPropagate);
    GetEventParameter(event, kEventParamControlOptimalBounds, typeHIRect,
                      0, sizeof(HIRect), 0, &optimalBounds);
    ReleaseEvent(event);
    return QSize(optimalBounds.size.width, optimalBounds.size.height);
}

bool MovieWidget::nativeControlsVisible() const
{
    OptionBits currentBits = HIMovieViewGetAttributes(HIViewRef(winId()));
    return currentBits & kHIMovieViewControllerVisibleAttribute;
}

void MovieWidget::setNativeControlsVisible(bool visible)
{
    if (visible != nativeControlsVisible()) {
        HIMovieViewChangeAttributes(HIViewRef(winId()),
                visible ? kHIMovieViewControllerVisibleAttribute : 0,
                !visible ? kHIMovieViewControllerVisibleAttribute : 0);
        updateGeometry();
        emit optimalSizeChange();
    }
}
